/*
 * Filename: tsignal_viewer.c
 * Date:     2022/02/05 22:58
 * Author:   Jan Faigl
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <float.h> //max float value

#include <unistd.h> //usleep

#include "xwin_otk_plot.h"

#ifndef MAX_PLOT_SIGNAL_LENGTH
#define MAX_PLOT_SIGNAL_LENGTH 1024  //maximal length shown in the plot 
#endif

#ifndef INIT_BUF_SIZE
#define INIT_BUF_SIZE 128
#endif

#ifndef REDRAW_DELAY_MS
#define REDRAW_DELAY_MS 5
#endif

static void my_assert(int r, const char *fcname, int line, const char *fname);

float *load_signal(FILE *in, unsigned int *size);
int draw_signal(const float *signal, int n);

enum { 
   ERROR_OK = EXIT_SUCCESS,
   ERROR_INPUT = 100, 
};
int main(int argc, char *argv[])
{
   int ret = EXIT_SUCCESS;
   float *signal = NULL;
   unsigned int n = 0;
   const char *fname = argc > 1 ? argv[1] : NULL;
   FILE *in = fname ? fopen(fname, "r") : stdin;
   if (!in) {
      fprintf(stderr, "ERROR: Cannot open input stream!\n");
      ret = ERROR_INPUT;
   } else {
      signal = load_signal(in, &n);
   }
   if (signal) {
      if (n > 0) {
	 ret = draw_signal(signal, n);
      }
      free(signal);
   }
   return ret;
}

// - function ----------------------------------------------------------------
int draw_signal(const float *signal, int n)
{
   int ret = ERROR_OK;
   int max_len = n > MAX_PLOT_SIGNAL_LENGTH ? MAX_PLOT_SIGNAL_LENGTH : n;
   float ticks[max_len]; // vla

   float min = FLT_MAX; 
   float max = -FLT_MAX;

   fprintf(stderr, "DEBUG: Draw signal with %d values using %d values\n", n, max_len);
   for (int i = 0; i < max_len; ++i) {
      ticks[i] = i;
   }
   for (int i = 0; i < n; ++i) {
      if (min > signal[i]) {
	 min = signal[i];
      }
      if (max < signal[i]) {
	 max = signal[i];
      }
   }
   fprintf(stderr, "DEBUG: Signal %d long has range [ %f, %f ]\n", n, min, max);

   xwin_init();
   xwin_set_plot(max_len, min, max);
   xwin_redraw_plot(max_len, ticks, signal);
   for (int i = 0; i < (n - max_len); ++i) {
      xwin_set_plot(max_len, min, max); // it removes the previous plot
      xwin_redraw_plot(max_len, ticks, &signal[i]);
      usleep(REDRAW_DELAY_MS * 1000);
   }
   xwin_close();
   return ret;
}

// - function ----------------------------------------------------------------
float *load_signal(FILE *in, unsigned int *size)
{
   int capacity = INIT_BUF_SIZE;
   int n = 0;
   bool ok = true;
   float *v = malloc(capacity * sizeof *v);
   my_assert(capacity > 0, __func__, __LINE__, __FILE__);
   my_assert(v != NULL, __func__, __LINE__, __FILE__);
   while (ok && !feof(in)) {
      if (n == capacity) {
	 float *t = realloc(v, 2 * capacity * sizeof *t);
	 my_assert(t != NULL, __func__, __LINE__, __FILE__);
	 capacity *= 2;
	 v = t;
      }
      int r = fscanf(in, "%f", &(v[n]));
      if (r == 0 || (r == EOF && !feof(in))) { //error in conversion or reading
	 n = 0;
	 free(v);
	 v = NULL;
	 ok = false;
      } else if (r == 1) {
	 n += 1; //increase number of read values
      }
   } //end while
   *size = n;
   fprintf(stderr, "DEBUG: Loaded signal size: %d\n", n);
   return v;
}

// - function ----------------------------------------------------------------
void my_assert(int r, const char *fcname, int line, const char *fname)
{
   if (!(r)) {
      fprintf(stderr, "ERROR: my_assert FAIL: %s() line %d in %s\n", fcname, line, fname);
      exit(-1);
   }
}

/* end of tsignal_viewer.c */
